"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const strictUnicode_1 = require("../../text/strictUnicode");
const textContainer_1 = require("../../text/textContainer");
class ArmPattern {
    constructor(firstChar, otherChar, lastChar, style) {
        this.firstChar = firstChar;
        this.otherChar = otherChar;
        this.lastChar = lastChar;
        this.style = style;
    }
    /**
     * Generates line with drawn armPlainLine like: `╰───╸`
     * with repeating `this.other` character to fit `this.armWidth`
     *
     * @remarks
     *
     * During generation `first` have higher priority then `last`.
     * So `first` will be the only generated character if `this.armWidth = 1`.
     * Same way `other` have lowest priority. And appars only if `this.armWidth > 2`
     *
     * @param {armWidthT} armWidth
     * Width of generated armPlainLine. Measured in monospaced-character positions.
     * @returns {StrictUnicodeLine}
     * Drawn armPlainLine, wrapped in StrictUnicodeLine.
     *
     * @remarks
     *
     * Returning StrictUnicodeLine instead of raw string is to skip
     * safaty-checks in tree generator/renderer.
     *
     * @memberof BasicPattern
     */
    generateArmPlainLine(armWidth) {
        const spacedArmWidth = (typeof armWidth === 'number')
            ? {
                preSpace: 0,
                arm: armWidth,
                postSpace: 0,
            }
            : armWidth;
        let generated = '';
        if (spacedArmWidth.preSpace > 0) {
            generated += ' '.repeat(spacedArmWidth.preSpace);
        }
        if (spacedArmWidth.arm > 0) {
            generated += this.firstChar.toString();
        }
        if (spacedArmWidth.arm > 2) {
            generated += this.otherChar.toString().repeat(spacedArmWidth.arm - 2);
        }
        if (spacedArmWidth.arm > 1) {
            generated += this.lastChar.toString();
        }
        if (spacedArmWidth.postSpace > 0) {
            generated += ' '.repeat(spacedArmWidth.postSpace);
        }
        return new strictUnicode_1.StrictUnicodeLine(generated, true);
    }
    generateArm(armWidth) {
        const plainLine = this.generateArmPlainLine(armWidth);
        const arm = new textContainer_1.AtomicTextContainer(plainLine, this.style);
        return arm;
    }
    static fromChars(chars, style) {
        const armPlainLinePattern = new ArmPattern(new strictUnicode_1.StrictUnicodeChar(chars[0]), new strictUnicode_1.StrictUnicodeChar(chars[1]), new strictUnicode_1.StrictUnicodeChar(chars[2]), style);
        return armPlainLinePattern;
    }
}
exports.ArmPattern = ArmPattern;
//# sourceMappingURL=armPattern.js.map