"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class TextContainerBase {
}
exports.TextContainerBase = TextContainerBase;
class AtomicTextContainer extends TextContainerBase {
    constructor(text, style) {
        super();
        this.text = text;
        this.style = style;
        this.size = this.precalcSize();
    }
    precalcSize() {
        const feedLines = this.text.splitToFeedLines();
        const firstLine = feedLines[0];
        const lastLine = feedLines[feedLines.length - 1];
        let otherLineMaxWidth = 0;
        for (let lineId = 1; lineId < feedLines.length - 1; lineId++) {
            const otherLine = feedLines[lineId];
            const otherLineWidth = otherLine.calcWidth();
            otherLineMaxWidth = Math.max(otherLineMaxWidth, otherLineWidth);
        }
        const size = {
            width: {
                first: firstLine.calcWidth(),
                last: lastLine.calcWidth(),
                other: otherLineMaxWidth,
            },
            heigth: feedLines.length - 1,
        };
        return size;
    }
    calcSize() {
        return this.size;
    }
    toString() {
        return this.text.toString();
    }
    flatten(initialStyle) {
        const style = this.style || initialStyle;
        return new FlatNonatomicTextContainer([new AtomicTextContainer(this.text, style)]);
    }
    splitToFlatFeedLines() {
        return this.text.splitToFeedLines().map((line) => {
            return new FlatNonatomicTextContainer([new AtomicTextContainer(line, this.style)]);
        });
    }
    *managedWrap(maxWidth) {
        const lineWrapGenerator = this.text.managedWrap(maxWidth);
        let done = false;
        while (true) {
            const generatorResult = lineWrapGenerator.next(maxWidth);
            done = generatorResult.done;
            const wrappedLine = generatorResult.value;
            const wrappedLineContainer = new AtomicTextContainer(wrappedLine, this.style);
            if (!done) {
                maxWidth = yield wrappedLineContainer;
            }
            else {
                return wrappedLineContainer;
            }
        }
    }
}
exports.AtomicTextContainer = AtomicTextContainer;
class NonatomicTextContainer extends TextContainerBase {
    constructor(children, style) {
        super();
        this.children = children;
        this.style = style;
    }
    calcSize() {
        const initialSize = {
            heigth: 0,
            width: {
                first: 0,
                other: 0,
                last: 0,
            }
        };
        const reduceState = this.children.reduce((reduceState, child) => {
            const currentSize = child.calcSize();
            // debugger; // TODO check is 0 or 1 needed for noline
            reduceState.linePadding += currentSize.width.first;
            if (reduceState.isFirst) {
                reduceState.size.width.first = reduceState.linePadding;
            }
            reduceState.size.width.other = Math.max(reduceState.size.width.other, currentSize.width.other, reduceState.linePadding);
            reduceState.size.heigth += currentSize.heigth;
            if (currentSize.heigth > 0) {
                reduceState.isFirst = false;
                reduceState.linePadding = reduceState.size.width.last;
            }
            return reduceState;
        }, { size: initialSize, linePadding: 0, isFirst: true });
        reduceState.size.width.last = reduceState.linePadding;
        return reduceState.size;
    }
    toString() {
        return this.children.reduce((text, child) => { return text + child.toString(); }, '');
    }
    flatten(initialStyle) {
        const style = this.style || initialStyle;
        const flatChildren = this.children.reduce((flatChildren, child) => {
            flatChildren = flatChildren.concat(child.flatten(style).children);
            return flatChildren;
        }, []);
        return new FlatNonatomicTextContainer(flatChildren);
    }
    splitToFlatFeedLines() {
        return this.flatten().splitToFlatFeedLines();
    }
    *managedWrap(maxWidth) {
        let remains = undefined;
        let remainsWidth = 0;
        if (this.children.length === 0) {
            return new NonatomicTextContainer([], this.style);
        }
        for (const child of this.children) {
            const lineWrapGenerator = child.managedWrap(maxWidth - remainsWidth);
            let done = false;
            while (!done) {
                const generatorResult = lineWrapGenerator.next(maxWidth - remainsWidth);
                done = generatorResult.done;
                const wrappedLine = generatorResult.value;
                if (!done) {
                    if (remains) {
                        const wrappedLineWithRemains = remains.concat([wrappedLine]);
                        maxWidth = yield new NonatomicTextContainer(wrappedLineWithRemains, this.style);
                    }
                    else {
                        maxWidth = yield new NonatomicTextContainer([wrappedLine], this.style);
                    }
                    remains = undefined;
                    remainsWidth = 0;
                }
                else {
                    if (remains) {
                        const wrappedLineWithRemains = remains.concat([wrappedLine]);
                        remains = wrappedLineWithRemains;
                        remainsWidth += wrappedLine.calcSize().width.first;
                    }
                    else {
                        remains = [wrappedLine];
                        remainsWidth = wrappedLine.calcSize().width.first;
                    }
                }
            }
        }
        // tail
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return new NonatomicTextContainer(remains, this.style);
    }
}
exports.NonatomicTextContainer = NonatomicTextContainer;
class FlatNonatomicTextContainer extends NonatomicTextContainer {
    constructor(children) {
        super(children, undefined);
        this.children = children;
    }
    splitToFlatFeedLines() {
        const flatFeedLines = [];
        let currentFeedLineAsAtomics = [];
        this.children.forEach((child) => {
            const childFeedLines = child.text.splitToFeedLines();
            const childFeedLinesAsAtomics = childFeedLines.map((childLine) => {
                const childLineAsAtomic = new AtomicTextContainer(childLine, child.style);
                return childLineAsAtomic;
            });
            if (childFeedLinesAsAtomics.length > 0) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                currentFeedLineAsAtomics.push(childFeedLinesAsAtomics.shift());
            }
            while (childFeedLinesAsAtomics.length > 0) {
                flatFeedLines.push(new FlatNonatomicTextContainer(currentFeedLineAsAtomics));
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                currentFeedLineAsAtomics = [childFeedLinesAsAtomics.pop()];
            }
        });
        // tail
        flatFeedLines.push(new FlatNonatomicTextContainer(currentFeedLineAsAtomics));
        return flatFeedLines;
    }
}
exports.FlatNonatomicTextContainer = FlatNonatomicTextContainer;
//# sourceMappingURL=textContainer.js.map